// This file is part of OpenCV project.
// It is subject to the license terms in the LICENSE file found in the top-level directory
// of this distribution and at http://opencv.org/license.html.
//
// Copyright (C) 2018 Intel Corporation


#ifndef OPENCV_GAPI_GPU_CORE_API_HPP
#define OPENCV_GAPI_GPU_CORE_API_HPP
/** @file
* @deprecated Use <opencv2/gapi/ocl/core.hpp> instead.
*/

#include <opencv2/gapi/ocl/core.hpp>

namespace cv {
namespace gapi {
namespace core {
namespace gpu {
    using namespace ocl;
} // namespace gpu
} // namespace core
} // namespace gapi
} // namespace cv


#endif // OPENCV_GAPI_GPU_CORE_API_HPP
